/**
 * Copyright (c) 1996-2023, SR Research Ltd., All Rights Reserved
 *
 * For use by SR Research licencees only. Redistribution and use in source
 * and binary forms, with or without modification, are NOT permitted.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the distribution.
 *
 * Neither name of SR Research Ltd nor the name of contributors may be used
 * to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*******************************************************************************
 * EYELINK EXPT SUPPORT: SDL PLATFORM (Coppied from WIN32 PLATFORM)			   *
 * by Suganthan Subramaniam                                                    *
 * EYELINK II: 3 February 2003                                                 *
 * Provides text support for sdl							                   *
 * This module is for user applications.                                       *
 *                                                                             *
 **************************************** WARNING ******************************
 *                                                                             *
 * UNDER NO CIRCUMSTANCES SHOULD PARTS OF THESE FILES BE COPIED OR COMBINED.   *
 * This will make your code impossible to upgrade to new releases in the future*
 * and SR Research will not give tech support for reorganized code.            *
 *                                                                             *
 * This file should not be modified. If you must modify it, copy the entire    *
 * file with a new name, and change the the new file.                          *
 *                                                                             *
 *******************************************************************************/


#define  _CRT_SECURE_NO_WARNINGS // for msvc, otherwise we get a lot of warnings.
#include <stdio.h>    // for vsprintf() function
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <SDL2/SDL.h>
#include <errno.h>

#ifdef __APPLE__ //MACOSX
#include <SDL2_ttf/SDL_ttf.h>
#include <CoreFoundation/CoreFoundation.h>
#include <NSSystemDirectories.h>
#else
#include <SDL2/SDL_ttf.h>
#endif
#include "fontmappings.h"
#include "sdl2_text_support.h"
#ifdef _WIN32
#include <direct.h>
#include <io.h>
#if (_MSC_VER > 1200)
#define strtok_r strtok_s
#else

#endif
#define makedir(a) _mkdir(a)
#define access(a,b) _access(a,b)
#define F_OK 00
#define PATH_DELIMITER ";"


#else
#include <libgen.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#define min(x,y) (x>y)? y:x
#define _stricmp strcasecmp
#define PATH_DELIMITER ":"
#define makedir(a) mkdir(a,S_IRWXU)
#endif

static  TTF_Font *font = NULL;
#define FONTSIZE 30
#define MAX_CHAR_PERLINE 1024
static int marginx =0;
static int marginy =0;
static int marginw =0;
static int marginh =0;
static double linespacing = (9.0/10.0);

/*
 search path is delimited by :
 */
static char font_search_path[4096]="";
static char output_folder_path[4096]="";
static char exe_folder_path[4096]="";

static FONT_MAP fontmap[] = FONTMAPPINGS;


void set_line_spacing(double linespace)
{
	linespacing = linespace;
}
void set_margin(int left, int right, int top, int bottom)
{
  marginx = left;
  marginy =top;
  marginw =right;
  marginh =bottom;
}



/*
 * Allow to add font search folder. By default fonts are search in
 * get_resources_path() and some system folders.
 * in windows <windir>\Fonts
 * in mac, we should technically search /System/Library/Fonts, /Library/Fonts and ~/Library/Fonts
 * However, in recent macs there does not seem to be any ttf files in these foledrs.  So they are not
 * searched.
 */
void add_font_search_path(const char *path)
{
	if(strlen(font_search_path)!=0)
	{
		strcat(font_search_path,PATH_DELIMITER);
	}
	strcat(font_search_path,path);
}

/*
 * Returns the path to the output folder.
 * For macos and linux, default output folder is ~/Documents/<exe name>_<date string>
 * For Windows default output folder is <user profile>/My Documents/<exe name>_<date string>
 */
const char * get_output_folder(void)
{
	return output_folder_path;
}

int set_output_folder(const char *path)
{
	strcpy(output_folder_path,path);
	return 0;
}



static int recursive_mkdir(const char *dir) {
	char tmp[4096];
	char *p = NULL;
	size_t len;
	int rv;


	sprintf(tmp,"%s",dir);
	len = strlen(tmp);
	if(tmp[len - 1] == '/')
		tmp[len - 1] = 0;
	for(p = tmp + 1; *p; p++)
		if(*p == '/') {

			*p = 0;
			rv =makedir(tmp);
			if(rv !=0)
			{
				if(errno != EEXIST)
				{
					printf("Failed to create folder %s %d\n", tmp, rv);
					return rv;
				}
			}
			*p = '/';
		}
	rv= makedir(tmp);
	if (rv != 0)
	{
		if (errno != EEXIST)
		{
			printf("Failed to create folder %s %d\n", tmp, rv);
			return rv;
		}
	}
	return 0;
}

#if defined(_WIN32)
#include <shlwapi.h>
#include <shlobj.h>
static const char * dirname(const char *path)
{
	static char result[2048];
	char drive[20];
	char dir[2048];
	_splitpath(path, drive, dir, NULL, NULL);
	if(drive[0])
		sprintf(result, "%s\\%s", drive, dir);
	else
		strcpy(result, dir);
	return result;
}
static const char * basename(const char *path)
{
	static char result[2048];
	_splitpath(path, NULL, NULL, result, NULL);
	return result;
}
static int getDocumentsFolder(char *doc)
{
	LPITEMIDLIST ppidl = NULL;
	HRESULT res = SHGetFolderLocation(NULL, CSIDL_MYDOCUMENTS, NULL, 0, &ppidl);
	if (res == S_OK)
	{
		SHGetPathFromIDList(ppidl, doc);
		ILFree(ppidl);
		return 1;
	}
	return 0;
}

static int getPublicDocumentsFolder(char *doc)
{
	LPITEMIDLIST ppidl = NULL;
	HRESULT res = SHGetFolderLocation(NULL, CSIDL_COMMON_DOCUMENTS, NULL, 0, &ppidl);
	if (res == S_OK)
	{
		SHGetPathFromIDList(ppidl, doc);
		ILFree(ppidl);
		return 1;
	}
	return 0;
}

static int try_creating_output_folder(const char *path, const char *exename, int make_sub)
{
	if(make_sub)
	{
		time_t current_time;
		struct tm * time_info;
		int len;
		char *outfolder;
		// output folder path prefix
		if (make_sub>1)
		{
			strcat(output_folder_path, "/Eyelink/SampleExperiments/");
			strcat(output_folder_path, exename);

		}
		strcat(output_folder_path, "/");
		//strcat(output_folder_path, exename); exe name alreay added
		// find the end fo the string to print time.
		len = strlen(output_folder_path);
		outfolder = output_folder_path + len;



		time(&current_time);
		time_info = localtime(&current_time);
		strftime(outfolder, sizeof(output_folder_path) - len, "%Y-%m-%d-%H-%M-%S", time_info);
	}
	return recursive_mkdir(output_folder_path);


}
int initialize_sdl_util(const char *exe_path, const char *output_folder, int create_output_folder)
{
	char exename[256];
	strcpy(exename,basename((char *)exe_path));
	strcpy(exe_folder_path, dirname((char *)exe_path));
	if(output_folder)
		strcpy(output_folder_path, output_folder);
	if(create_output_folder)
	{
		if(output_folder_path[0] && !try_creating_output_folder(output_folder_path, exename, 0)) // check the give output folder
		   return 1;
		memset(output_folder_path,0,sizeof(output_folder_path));
		if(_getcwd(output_folder_path,sizeof(output_folder_path))) // check current folder
		   {
			   if(output_folder_path[0] && !try_creating_output_folder(output_folder_path, exename,1))
				   return 1;
		   }
		memset(output_folder_path,0,sizeof(output_folder_path));
		if(getPublicDocumentsFolder(output_folder_path) && !try_creating_output_folder(output_folder_path, exename,2)) //check the public documents folder
			return 1;
		memset(output_folder_path,0,sizeof(output_folder_path));
		if(getDocumentsFolder(output_folder_path) && !try_creating_output_folder(output_folder_path, exename,2)) // chec the Documents folder
			return 1;
		return 0;
	}
	return 1;
}
#else
// for mac and linux
/*
 *initializes the output folder.
 *returns non zero on success. 0 if failed.
 */
int initialize_sdl_util(const char *_exe_path, const char *output_folder, int create_output_folder)
{
	char exe_path[4096]; //dirname and basename modifies the buffer
	char * homepath = getenv("HOME");
	strcpy(exe_path, _exe_path);
	strcpy(exe_folder_path, dirname((char *)exe_path));
	if(homepath)
	{

		if(output_folder)
		{
			strcpy(output_folder_path, output_folder);
			if(create_output_folder)
			{
				if(recursive_mkdir(output_folder_path))
					return 0;
			}
		}
		else
		{
			// output folder path prefix
			strcpy(output_folder_path, homepath);
			strcat(output_folder_path,"/Documents/Eyelink/SampleExperiments/");
			strcpy(exe_path, _exe_path);// reset the buffer
			strcat(output_folder_path,basename((char *)exe_path));
			strcat(output_folder_path,"/");

			// find the end fo the string to print time.
			int len =strlen(output_folder_path);
			char *outfolder =output_folder_path + len;
			time_t current_time;
			struct tm * time_info;


			time(&current_time);
			time_info = localtime(&current_time);
			strftime(outfolder, sizeof(output_folder_path)-len, "%Y-%m-%d-%H-%M-%S", time_info);
			if(create_output_folder)
			{
				if(recursive_mkdir(output_folder_path))
					return 0;
			}
		}

	}
	return 1;
}
#endif



/*
 tries to find the font file name and returns the file name.
 search_index takes in and return the index on the map.
 For the first call assign the value of 0. On the consequtive times
 keep with the last returned value.
 if none found, searh_index is set to -1;

 eg.
 searh_index=0;
 get_font_file_name(NULL, &searh_index);
 get_font_file_name(NULL, &searh_index); --> In this call the search continues from where we left off.

 */
static char * get_font_file_name(const char *_fontname, int *searh_index, int bold )
{
	char *fontname=(char *)_fontname; //to fool the compiler from complaing
	int i =*searh_index;
        if(!fontname) fontname= FONTNAME; // default font
	for(; fontmap[i].filename && fontmap[i].font_face && fontmap[i].font_name; i++)
	{
		if(_stricmp(fontmap[i].font_name,fontname) ==0)
		{
			*searh_index=i+1;
			return fontmap[i].filename;
		}
	}
	*searh_index=-1;
	return FONTNAME; // default font
}

/*
* This implementation is for the mac app. resource path is .app/Contents/Resources
*/
#if defined(__APPLE__) //defined(MACOSX) && defined(MACAPP)
const char * get_resources_path()
{
	static char wd[1024] = {0};
	if(!strlen(wd))
	{
		CFURLRef appRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
		CFStringRef appPath = CFURLCopyFileSystemPath(appRef,kCFURLPOSIXPathStyle);
		const char *pathPtr = CFStringGetCStringPtr(appPath,CFStringGetSystemEncoding());
		strcpy(wd,pathPtr);
		strcat(wd,"/Contents/Resources");
		return wd;
	}
	return wd;
}
#else
/* For windows and linux
*/

const char * get_resources_path()
{
	return exe_folder_path;
}
#endif

#if defined(_WIN32)
static const char * get_windows_font_path()
{
	static  char wd[1024] = {0};
	if(!strlen(wd))
	{
		strcpy(wd, getenv("windir"));
		strcat(wd,"/Fonts/");
	}
	return wd;
}
#endif
/*
 * returns 1 if file found in the path given
 * result contains the full path of the fille
 */
static int search_font_in_path(const char *fontname, const char *path, char * result)
{
	char fpath[2048];

	if (NULL == path)
	{
		if (NULL == fontname)
		{
			printf("Unable to search for null font in a null path.\n");
			return 0;
		}
		else
		{
			printf("Unable to search for font %s in a null path.\n", fontname);
			return 0;
		}
	}

	if(strlen(path))
		sprintf(fpath, "%s/%s",  path, fontname);
	else
		sprintf(fpath, "%s",  fontname);
	if( access( fpath, F_OK ) == 0 ) {
		strcpy(result, fpath);
		return 1;
	} else {
		return 0;
	}
}

static int search_all_font_in_path(const char *fontname,int bold, const char *path, char * result)
{
	int searh_index = 0;
	// not found search more.
	do
	{
		const char * fontfile = get_font_file_name(fontname, &searh_index, bold);
		// check current folder or abs path first.
		if (search_font_in_path(fontfile, path, result))
		{ // font file found.
			return 1;
		}
	} while (searh_index>0);
	return 0;
}

// look for font in the font search path.
static int search_for_font(char *fontname, int bold, char * result)
{

  char search_path_copy[4096] = {0};
  char* token;
  char* rest;

 	const char * searchfolder = NULL;
	if (search_all_font_in_path(fontname, bold, "", result))
		return 1;

	searchfolder = NULL;
#if defined(__APPLE__)
  // in macos we check for get_resources_path first, then any path given in the search path.
  searchfolder = get_resources_path();
#else
#if defined(_WIN32)
  searchfolder = get_windows_font_path();
#endif
#endif

  if (search_all_font_in_path(fontname, bold, searchfolder, result))
	  return 1;


// search in font search path
  rest = search_path_copy;
  strcpy(search_path_copy, font_search_path);

#if (_MSC_VER > 1200) || !defined(_WIN32)
  while ((token = strtok_r(rest, PATH_DELIMITER, &rest)))
  {
	  if (search_all_font_in_path(fontname, bold, token, result))
		  return 1;
  }
#else
// no multi-thread support
  token = strtok(rest, PATH_DELIMITER);
  if (search_all_font_in_path(fontname, bold, token, result))
	  return 1;
  while ((token = strtok(NULL, PATH_DELIMITER)))
  {
	  if (search_all_font_in_path(fontname, bold, token, result))
		  return 1;
  }
#endif
  return 0;
}
int get_new_font(char *fontname, int font_size, int bold)
{
	char fontpath[2048] = {0};

	if(!search_for_font(fontname, bold,fontpath))
		if(!search_for_font(fontname, 0,fontpath)) // try without bold setting.
		{
			printf("Failed to find font %s\n",  fontname);
			return 0;
		}


	//printf("fontpath for name is %s %s\n", fontpath, fontname);
	if(font) /* close the font first */
	{
		TTF_CloseFont(font);
		font = NULL;
	}
	  font = TTF_OpenFont(fontpath,font_size);
	  if(!font)
	  {
		  return -1;
	  }

	  if(bold)
		TTF_SetFontStyle(font,TTF_STYLE_BOLD);
	  return 0;
}



int get_font_height()
{
	if(font)
		return TTF_FontHeight(font);
	return -1;
}

static void adjustPlacement(SDL_Rect *rect, int scrwidth, int format)
{
	if(format & JUSTIFY_RIGHT)
		rect->x = scrwidth-rect->w;
	if(format & CENTER )
		rect->x = (scrwidth - rect->w)/2;
}

void sdl2_printf_ex(SDL_Window* window, char *text, int format,
					int x, int y, SDL_Color foreground)
{
	char temp [MAX_CHAR_PERLINE];
	SDL_Renderer * renderer = NULL;
	SDL_Surface *textsurface = NULL;
	SDL_Texture *texture = NULL;
	int scr_width = 0;
	int scr_height = 0;
	int width = 0;
	int height = 0;
	SDL_Rect rect;
	int stop = 0;
	int stop1 = 0;
	int stop2 = 0;
	int i = 0;

	// get font
	if(!font)
		get_new_font(FONTNAME,FONTSIZE,0);
	if(!font)
	{
		printf("Font cannot be loaded\n");
		return;
	}

	// set size
	TTF_SizeText(font, text, &width, &height);

	// get renderer
	renderer = SDL_GetRenderer(window);
	if (!renderer)
	{
		printf("Invalid window handle\n");
		return;
	}

	// get window width & height
	SDL_GetWindowSize(window, &scr_width, &scr_height);

	if(y + height + marginh > scr_height)
		return; // skip out of margin.

	memset(temp, 0, MAX_CHAR_PERLINE);
	if(x + width + marginw <= scr_width)
	{
		if(strlen(text) > MAX_CHAR_PERLINE)
			text[MAX_CHAR_PERLINE - 1] = 0; // chop the line.
		strcpy(temp, text);
		if(strlen(temp) ==0) return; // no text
		textsurface = TTF_RenderText_Blended(font, temp, foreground);
		if(!textsurface)
		{
			printf("Could not create text surface (%s)\n", SDL_GetError());
			return;
		}
		texture = SDL_CreateTextureFromSurface(renderer, textsurface);
		if (!texture)
		{
			printf("Could not create texture from surface (%s)\n", SDL_GetError());
			return;
		}
		SDL_FreeSurface(textsurface);

		rect.x = x;
		rect.y = y;
		SDL_QueryTexture(texture, NULL, NULL, &rect.w, &rect.h);
		adjustPlacement(&rect, scr_width, format);

		SDL_RenderCopy(renderer, texture, NULL, &rect);

		return;
	}

	// o/w (x + width + marginw > scr_width)
	strncpy(temp, text, MAX_CHAR_PERLINE - 1);

	// look for \n or \r
	for(i = 0; i < MAX_CHAR_PERLINE - 1; i++)
	{
		if(temp[i] == '\n' || temp[i] == '\r')
		{
			stop1 = i;
			break;
		}
	}
	if(!stop)
	{
		// no line breaks found. break at a white space
		int i =0;
		for(i = MAX_CHAR_PERLINE-2; i >=0 ; i--)
		{
			int w =0,
				h =0;
			char last = temp[i+1];
			temp[i+1] = 0;
			TTF_SizeText(font, temp,&w, &h);
			if( (temp[i] == ' ' || temp[i] == '\t') && (w + x + marginw < scr_width))
			{
				stop2 = i;
				break;
			}
			temp[i+1] = last;
		}
	}

	if(stop1 && stop2)
		stop = min(stop1, stop2);
	else if(stop1)
		stop = stop1;
	else if(stop2)
		stop = stop2;

	if(stop)
	{
		temp[stop] = 0;
		if(strlen(temp) ==0) return; // no text
		textsurface = TTF_RenderText_Blended(font, temp, foreground);
		if(!textsurface)
		{
			printf("Could not create text surface (%s)\n", SDL_GetError());
			return;
		}

		texture = SDL_CreateTextureFromSurface(renderer, textsurface);
		if (!texture)
		{
			printf("Could not create texture from surface (%s)\n", SDL_GetError());
			return;
		}

		SDL_FreeSurface(textsurface);

		rect.x = x;
		rect.y = y;
		SDL_QueryTexture(texture, NULL, NULL, &rect.w, &rect.h);
		adjustPlacement(&rect, scr_width, format);

		SDL_RenderCopy(renderer, texture, NULL, &rect);

		if(format & WRAP && strlen(temp) && strlen(temp) < strlen(text))
		{
			int i = stop+1;
			if(stop != stop1) // line break dont skip the spaces
			{
				for(;i < MAX_CHAR_PERLINE -1; i++)
				{
					if(temp[i] != ' ')
						break;
				}
			}
			sdl2_printf_ex(window, text+i, format, x, y+(int)((double)height*(linespacing)), foreground);
		}
	}
}


/*******************************************************************************
 * sdl_demo_printf
 * Purpose: printf like text printing routine to a surface.
 * Input  : surface
 *			x  - x position on the resource
 *			y  - y position on the resource.
 *          fsize - font size
 *          ftype - font type
 *			text - format string
 *			...  - arguments
 * Output :
 *******************************************************************************/
//void graphic_printf(SDL_Surface *surface, SDL_Color fg, int format, int x, int y, const char *fmt, ...)
//{
//	va_list argptr;
//	char *msgbuf = malloc(sizeof(char) *1024*4);
//
//
//	if(format & WRAP)
//	{
//		x+= marginx;
//		y+= marginy;
//	}
//	va_start(argptr, fmt);          // Create the text string, similar to printf()
//	vsprintf(msgbuf, fmt, argptr);
//	va_end(argptr);
//	sdl_printf(surface,msgbuf,format, x, y, fg);
//	free(msgbuf);
//}

void sdl2_printf(SDL_Window *window, SDL_Color fg, int format, int x, int y, const char *fmt, ...)
{
	va_list argptr;
	char *msgbuf = malloc(sizeof(char) * 1024 * 4);

	if(format & WRAP)
	{
		x+= marginx;
		y+= marginy;
	}
	va_start(argptr, fmt);
	vsprintf(msgbuf, fmt, argptr);
	va_end(argptr);
	sdl2_printf_ex(window, msgbuf, format, x, y, fg);
	free(msgbuf);
}

#ifdef _WIN32
static int OSCopyFile(const char* source, const char* destination)
{
	BOOL cancel = FALSE;
	return CopyFileEx(source, destination, NULL, NULL, &cancel, COPY_FILE_FAIL_IF_EXISTS);

}
#else
#if defined(__APPLE__) || defined(__FreeBSD__)
#include <copyfile.h>
#else
#include <sys/sendfile.h>
#endif

// copied from https://stackoverflow.com/questions/2180079/how-can-i-copy-a-file-on-unix-using-c
static int OSCopyFile(const char* source, const char* destination)
{
	int input, output;
	if ((input = open(source, O_RDONLY)) == -1)
	{
		printf("failed to open source file '%s' \n",  source);
		return -1;
	}
	if ((output = creat(destination, 0660)) == -1)
	{
		printf("failed to open destination file '%s'\n",  destination);
		close(input);
		return -1;
	}

	//Here we use kernel-space copying for performance reasons
#if defined(__APPLE__) || defined(__FreeBSD__)
	//fcopyfile works on FreeBSD and OS X 10.5+
	int result = fcopyfile(input, output, 0, COPYFILE_ALL);
#else
	//sendfile will work with non-socket output (i.e. regular file) on Linux 2.6.33+
	off_t bytesCopied = 0;
	struct stat fileinfo = {0};
	fstat(input, &fileinfo);
	int result = sendfile(output, input, &bytesCopied, fileinfo.st_size);
	if(result<0)
		printf("sendfile failed!. Error:%d Source:%s Destination:%s\n", result, source, destination);
#endif

	close(input);
	close(output);

	return result;
}
#endif





int copy_resource_to_output_folder(const char *resource)
{
	// for now stub. set the following link for more copy options.
	//https://stackoverflow.com/questions/2180079/how-can-i-copy-a-file-on-unix-using-c
	char src[2048];
	char dest[2048];
	const char *destfolder;
	sprintf(src,"%s/%s", get_resources_path(), resource);
	sprintf(dest,"%s/%s", get_output_folder(), resource);

	destfolder = dirname(dest);
	recursive_mkdir(destfolder);
	// recopy. the dirname reset the buffers.
	sprintf(src,"%s/%s", get_resources_path(), resource);
	sprintf(dest,"%s/%s", get_output_folder(), resource);
	return OSCopyFile(src,dest);
}

int open_output_folder(void)
{
#if defined(__APPLE__)
	char opencmd[2048];
	sprintf(opencmd,"open %s", get_output_folder());
	return system(opencmd);
#elif defined(_WIN32)
	return (int)ShellExecute(NULL, NULL, get_output_folder(), NULL, NULL, SW_SHOWNORMAL);
#else
	char opencmd[2048];
	printf("Output folder: %s \n",   get_output_folder());
	sprintf(opencmd,"xdg-open %s", get_output_folder());
	return system(opencmd);
#endif
}
